@@ -1,15 +1,29 @@
 import time
-from fontTools.ttLib import TTFont, TTLibError
-from fontTools.ttLib.tables._h_e_a_d import mac_epoch_diff
-from fontTools.misc.textTools import num2binary
 from fontTools.pens.boundsPen import ControlBoundsPen
+from fontTools.pens.hashPointPen import HashPointPen
+from fontTools.ttLib import TTFont, TTLibError
+from fontTools.ttLib.tables._g_l_y_f import (
+    OVERLAP_COMPOUND,
+    ROUND_XY_TO_GRID,
+    USE_MY_METRICS,
+)
+from fontTools.ttLib.tables._h_e_a_d import mac_epoch_diff
 from extractor.exceptions import ExtractorError
+from extractor.stream import InstructionStream
 from extractor.tools import RelaxedInfo, copyAttr
 
+
+TRUETYPE_INSTRUCTIONS_KEY = "public.truetype.instructions"
+TRUETYPE_ROUND_KEY = "public.truetype.roundOffsetToGrid"
+TRUETYPE_METRICS_KEY = "public.truetype.useMyMetrics"
+TRUETYPE_OVERLAP_KEY = "public.truetype.overlap"
+OBJECT_LIBS_KEY = "public.objectLibs"
+
 # ----------------
 # Public Functions
 # ----------------
 
+
 def isOpenType(pathOrFile):
     try:
         font = TTFont(pathOrFile)
@@ -18,12 +32,23 @@ def isOpenType(pathOrFile):
         return False
     return True
 
-def extractFontFromOpenType(pathOrFile, destination, doGlyphOrder=True, doGlyphs=True, doInfo=True, doKerning=True, customFunctions=[]):
+
+def extractFontFromOpenType(
+    pathOrFile,
+    destination,
+    doGlyphOrder=True,
+    doGlyphs=True,
+    doInfo=True,
+    doKerning=True,
+    customFunctions=[],
+    doInstructions=True,
+):
     source = TTFont(pathOrFile)
     if doInfo:
         extractOpenTypeInfo(source, destination)
     if doGlyphs:
         extractOpenTypeGlyphs(source, destination)
+        extractUnicodeVariationSequences(source, destination)
     if doGlyphOrder:
         extractGlyphOrder(source, destination)
     if doKerning:
@@ -33,17 +58,172 @@ def extractFontFromOpenType(pathOrFile, destination, doGlyphOrder=True, doGlyphs
         destination.kerning.update(kerning)
     for function in customFunctions:
         function(source, destination)
+    if doInstructions:
+        extractInstructions(source, destination)
     source.close()
 
+
 def extractGlyphOrder(source, destination):
     glyphOrder = source.getGlyphOrder()
     if len(glyphOrder):
         destination.lib["public.glyphOrder"] = glyphOrder
 
+
+# ---------------------------
+# Unicode Variation Sequences
+# ---------------------------
+
+
+def extractUnicodeVariationSequences(source, destination):
+    """
+    Extract the Unicode Variation Sequences
+    """
+    cmap = source.get("cmap")
+    mapping = cmap.getBestCmap()
+    for subtable in cmap.tables:
+        if subtable.format == 14:
+            destination.lib["public.unicodeVariationSequences"] = {
+                "%04X" % variationSelector: {
+                    "%04X" % charValue: glyphName if glyphName else mapping.get(charValue)
+                    for (charValue, glyphName) in uvsList
+                }
+                for variationSelector, uvsList in subtable.uvsDict.items()
+            }
+
+
+# ------------
+# Instructions
+# ------------
+
+
+def extractInstructions(source, destination):
+    if "glyf" not in source:
+        return
+
+    lib = destination.lib[TRUETYPE_INSTRUCTIONS_KEY] = {
+        "formatVersion": 1,
+        "maxFunctionDefs": 0,
+        "maxInstructionDefs": 0,
+        "maxStackElements": 0,
+        "maxStorage": 0,
+        "maxTwilightPoints": 0,
+        "maxZones": 0,
+    }
+    extractControlValues(source, lib)
+    extractFontProgram(source, lib)
+    extractGlyphPrograms(source, destination)
+    extractMaxpValues(source, lib)
+    extractPreProgram(source, lib)
+
+
+def extractControlValues(source, lib):
+    """
+    Extract the TrueType control values table to the font lib.
+    """
+    if "cvt " not in source:
+        return
+    cvt = source["cvt "]
+    lib["controlValue"] = [
+        {"id": str(i), "value": val} for i, val in enumerate(cvt.values)
+    ]
+
+
+def extractFontProgram(source, lib):
+    """
+    Extract the TrueType font program to the font lib.
+    """
+    if "fpgm" not in source:
+        return
+    fpgm = source["fpgm"].program
+    lib["fontProgram"] = _byteCodeToTtxAssembly(fpgm)
+
+
+def extractGlyphPrograms(source, destination):
+    """
+    Extract the TrueType pre-program to the font lib.
+    """
+    if "glyf" not in source:
+        return
+    glyph_set = source.getGlyphSet()
+    for name in glyph_set.keys():
+        glyph = glyph_set[name]._glyph
+        dest_glyph = destination[name]
+        if glyph.isComposite():
+            # Extract composite flags
+            _extractCompositeFlags(glyph, dest_glyph)
+        if not hasattr(glyph, "program"):
+            continue
+
+        hash_pen = HashPointPen(dest_glyph.width, destination)
+        dest_glyph.drawPoints(hash_pen)
+        lib = dest_glyph.lib[TRUETYPE_INSTRUCTIONS_KEY] = {
+            "formatVersion": "1",
+            "id": hash_pen.hash,
+        }
+        lib["assembly"] = _byteCodeToTtxAssembly(glyph.program)
+
+
+def extractMaxpValues(source, lib):
+    """
+    Extract the TrueType maximum profile values to the font lib.
+    """
+    if "maxp" not in source:
+        return
+    maxp = source["maxp"]
+    lib.update(
+        {
+            "maxFunctionDefs": maxp.maxFunctionDefs,
+            "maxInstructionDefs": maxp.maxInstructionDefs,
+            "maxStackElements": maxp.maxStackElements,
+            "maxStorage": maxp.maxStorage,
+            "maxTwilightPoints": maxp.maxTwilightPoints,
+            "maxZones": maxp.maxZones,
+        }
+    )
+
+
+def extractPreProgram(source, lib):
+    """
+    Extract the TrueType pre-program to the font lib.
+    """
+    if "prep" not in source:
+        return
+    prep = source["prep"].program
+    lib["controlValueProgram"] = _byteCodeToTtxAssembly(prep)
+
+
+def _byteCodeToTtxAssembly(program):
+    stream = InstructionStream(program_bytes=program.getBytecode())
+    return "\n%s\n" % str(stream)
+
+
+def _extractCompositeFlags(glyph, dest_glyph):
+    # Find the lib key or add it
+    if OBJECT_LIBS_KEY not in dest_glyph.lib:
+        dest_glyph.lib[OBJECT_LIBS_KEY] = {}
+    object_libs = dest_glyph.lib[OBJECT_LIBS_KEY]
+
+    for ci, c in enumerate(glyph.components):
+        flags = {}
+        flags[TRUETYPE_ROUND_KEY] = bool(c.flags & ROUND_XY_TO_GRID)
+        flags[TRUETYPE_METRICS_KEY] = bool(c.flags & USE_MY_METRICS)
+
+        if flags:
+            identifier = f"component{ci}"
+            dest_glyph.components[ci].identifier = identifier
+            object_libs[identifier] = flags
+
+        # Overlap is stored directly in the glyph lib
+        dest_glyph.lib["public.truetype.overlap"] = bool(
+            c.flags & OVERLAP_COMPOUND
+        )
+
+
 # ----
 # Info
 # ----
 
+
 def extractOpenTypeInfo(source, destination):
     info = RelaxedInfo(destination.info)
     _extractInfoHead(source, info)
@@ -55,6 +235,7 @@ def extractOpenTypeInfo(source, destination):
     _extractInfoCFF(source, info)
     _extractInfoGasp(source, info)
 
+
 def _extractInfoHead(source, info):
     head = source["head"]
     # version
@@ -84,6 +265,7 @@ def _extractInfoHead(source, info):
         styleMapStyleName = "italic"
     info.styleMapStyleName = styleMapStyleName
 
+
 def _extractInfoName(source, info):
     records = []
     nameIDs = {}
@@ -94,15 +276,21 @@ def _extractInfoName(source, info):
         languageID = record.langID
         string = record.toUnicode()
         nameIDs[nameID, platformID, encodingID, languageID] = string
-        records.append((nameID, platformID, encodingID, languageID,
-            dict(
-                nameID=nameID,
-                platformID=platformID,
-                encodingID=encodingID,
-                languageID=languageID,
-                string=string
+        records.append(
+            (
+                nameID,
+                platformID,
+                encodingID,
+                languageID,
+                dict(
+                    nameID=nameID,
+                    platformID=platformID,
+                    encodingID=encodingID,
+                    languageID=languageID,
+                    string=string,
+                ),
             )
-        ))
+        )
     attributes = dict(
         familyName=_priorityOrder(16) + _priorityOrder(1),
         styleName=_priorityOrder(17) + _priorityOrder(2),
@@ -124,7 +312,7 @@ def _extractInfoName(source, info):
         openTypeNameCompatibleFullName=_priorityOrder(18),
         openTypeNameSampleText=_priorityOrder(20),
         openTypeNameWWSFamilyName=_priorityOrder(21),
-        openTypeNameWWSSubfamilyName=_priorityOrder(22)
+        openTypeNameWWSSubfamilyName=_priorityOrder(22),
     )
     for attr, priority in attributes.items():
         value = _skimNameIDs(nameIDs, priority)
@@ -132,14 +320,16 @@ def _extractInfoName(source, info):
             setattr(info, attr, value)
     info.openTypeNameRecords = [record[-1] for record in sorted(records)]
 
+
 def _priorityOrder(nameID):
     priority = [
         (nameID, 1, 0, 0),
         (nameID, 1, None, None),
-        (nameID, None, None, None)
+        (nameID, None, None, None),
     ]
     return priority
 
+
 def _skimNameIDs(nameIDs, priority):
     for (nameID, platformID, platEncID, langID) in priority:
         for (nID, pID, pEID, lID), text in nameIDs.items():
@@ -153,6 +343,7 @@ def _skimNameIDs(nameIDs, priority):
                 continue
             return text
 
+
 def _extracInfoOS2(source, info):
     os2 = source["OS/2"]
     # openTypeOS2WidthClass
@@ -186,14 +377,21 @@ def _extracInfoOS2(source, info):
             os2.panose.bArmStyle,
             os2.panose.bLetterForm,
             os2.panose.bMidline,
-            os2.panose.bXHeight
+            os2.panose.bXHeight,
         ]
     # openTypeOS2FamilyClass
     # XXX info.openTypeOS2FamilyClass
     if hasattr(os2, "ulUnicodeRange1"):
-        info.openTypeOS2UnicodeRanges = binaryToIntList(os2.ulUnicodeRange1) + binaryToIntList(os2.ulUnicodeRange2, 32) + binaryToIntList(os2.ulUnicodeRange3, 64) + binaryToIntList(os2.ulUnicodeRange4, 96)
+        info.openTypeOS2UnicodeRanges = (
+            binaryToIntList(os2.ulUnicodeRange1)
+            + binaryToIntList(os2.ulUnicodeRange2, 32)
+            + binaryToIntList(os2.ulUnicodeRange3, 64)
+            + binaryToIntList(os2.ulUnicodeRange4, 96)
+        )
     if hasattr(os2, "ulCodePageRange1"):
-        info.openTypeOS2CodePageRanges = binaryToIntList(os2.ulCodePageRange1) + binaryToIntList(os2.ulCodePageRange2, 32)
+        info.openTypeOS2CodePageRanges = binaryToIntList(
+            os2.ulCodePageRange1
+        ) + binaryToIntList(os2.ulCodePageRange2, 32)
     copyAttr(os2, "sxHeight", info, "xHeight")
     copyAttr(os2, "sCapHeight", info, "capHeight")
     copyAttr(os2, "sTypoAscender", info, "ascender")
@@ -216,6 +414,7 @@ def _extracInfoOS2(source, info):
     copyAttr(os2, "yStrikeoutSize", info, "openTypeOS2StrikeoutSize")
     copyAttr(os2, "yStrikeoutPosition", info, "openTypeOS2StrikeoutPosition")
 
+
 def _extractInfoHhea(source, info):
     if "hhea" not in source:
         return
@@ -227,6 +426,7 @@ def _extractInfoHhea(source, info):
     info.openTypeHheaCaretSlopeRun = hhea.caretSlopeRun
     info.openTypeHheaCaretOffset = hhea.caretOffset
 
+
 def _extractInfoVhea(source, info):
     if "vhea" not in source:
         return
@@ -239,6 +439,7 @@ def _extractInfoVhea(source, info):
     if hasattr(vhea, "caretOffset"):
         info.openTypeVheaCaretOffset = vhea.caretOffset
 
+
 def _extractInfoPost(source, info):
     post = source["post"]
     info.italicAngle = post.italicAngle
@@ -246,6 +447,7 @@ def _extractInfoPost(source, info):
     info.postscriptUnderlinePosition = post.underlinePosition
     info.postscriptIsFixedPitch = bool(post.isFixedPitch)
 
+
 def _extractInfoCFF(source, info):
     if "CFF " not in source:
         return
@@ -262,18 +464,25 @@ def _extractInfoCFF(source, info):
         info.postscriptBlueValues = private.rawDict.get("BlueValues", [])
         info.postscriptOtherBlues = private.rawDict.get("OtherBlues", [])
         info.postscriptFamilyBlues = private.rawDict.get("FamilyBlues", [])
-        info.postscriptFamilyOtherBlues = private.rawDict.get("FamilyOtherBlues", [])
+        info.postscriptFamilyOtherBlues = private.rawDict.get(
+            "FamilyOtherBlues", []
+        )
         info.postscriptStemSnapH = private.rawDict.get("StemSnapH", [])
         info.postscriptStemSnapV = private.rawDict.get("StemSnapV", [])
         info.postscriptBlueFuzz = private.rawDict.get("BlueFuzz", None)
         info.postscriptBlueShift = private.rawDict.get("BlueShift", None)
         info.postscriptBlueScale = private.rawDict.get("BlueScale", None)
         info.postscriptForceBold = bool(private.rawDict.get("ForceBold", None))
-        info.postscriptNominalWidthX = private.rawDict.get("nominalWidthX", None)
-        info.postscriptDefaultWidthX = private.rawDict.get("defaultWidthX", None)
+        info.postscriptNominalWidthX = private.rawDict.get(
+            "nominalWidthX", None
+        )
+        info.postscriptDefaultWidthX = private.rawDict.get(
+            "defaultWidthX", None
+        )
     # XXX postscriptSlantAngle
     # XXX postscriptUniqueID
 
+
 def _extractInfoGasp(source, info):
     if "gasp" not in source:
         return
@@ -289,15 +498,14 @@ def _extractInfoGasp(source, info):
             behavior.append(2)
         if bits & 0x0008:
             behavior.append(3)
-        record = dict(
-            rangeMaxPPEM=size,
-            rangeGaspBehavior=behavior
-        )
+        record = dict(rangeMaxPPEM=size, rangeGaspBehavior=behavior)
         records.append(record)
     info.openTypeGaspRangeRecords = records
 
+
 # Tools
 
+
 def binaryToIntList(value, start=0):
     intList = []
     counter = start
@@ -308,15 +516,16 @@ def binaryToIntList(value, start=0):
         counter += 1
     return intList
 
+
 # --------
 # Outlines
 # --------
 
+
 def extractOpenTypeGlyphs(source, destination):
     # grab the cmap
     vmtx = source.get("vmtx")
     vorg = source.get("VORG")
-    cmap = source.getBestCmap()
     is_ttf = "glyf" in source
     reversedMapping = source.get("cmap").buildReversed()
     # grab the glyphs
@@ -354,10 +563,12 @@ def extractOpenTypeGlyphs(source, destination):
         # unicodes
         destinationGlyph.unicodes = reversedMapping.get(glyphName, [])
 
+
 # -------
 # Kerning
 # -------
 
+
 def extractOpenTypeKerning(source, destination):
     kerning = {}
     groups = {}
@@ -370,19 +581,26 @@ def extractOpenTypeKerning(source, destination):
         groups[name] = list(sorted(group))
     return kerning, groups
 
+
 def _extractOpenTypeKerningFromGPOS(source):
     gpos = source["GPOS"].table
     # get an ordered list of scripts
     scriptOrder = _makeScriptOrder(gpos)
     # extract kerning and classes from each applicable lookup
-    kerningDictionaries, leftClassDictionaries, rightClassDictionaries = _gatherDataFromLookups(gpos, scriptOrder)
+    (
+        kerningDictionaries,
+        leftClassDictionaries,
+        rightClassDictionaries,
+    ) = _gatherDataFromLookups(gpos, scriptOrder)
     # merge all kerning pairs
     kerning = _mergeKerningDictionaries(kerningDictionaries)
     # get rid of groups that have only one member
     leftSingleMemberGroups = _findSingleMemberGroups(leftClassDictionaries)
     rightSingleMemberGroups = _findSingleMemberGroups(rightClassDictionaries)
     # filter out the single glyph groups from the kerning
-    kerning = _removeSingleMemberGroupReferences(kerning, leftSingleMemberGroups, rightSingleMemberGroups)
+    kerning = _removeSingleMemberGroupReferences(
+        kerning, leftSingleMemberGroups, rightSingleMemberGroups
+    )
     # merge groups that have the exact same member list
     leftClasses, leftClassRename = _mergeClasses(leftClassDictionaries)
     rightClasses, rightClassRename = _mergeClasses(rightClassDictionaries)
@@ -390,12 +608,16 @@ def _extractOpenTypeKerningFromGPOS(source):
     _validateClasses(leftClasses)
     _validateClasses(rightClasses)
     # populate the class marging into the kerning
-    kerning = _replaceRenamedPairMembers(kerning, leftClassRename, rightClassRename)
+    kerning = _replaceRenamedPairMembers(
+        kerning, leftClassRename, rightClassRename
+    )
     # rename the groups to final names
     leftClassRename = _renameClasses(leftClasses, "public.kern1.")
     rightClassRename = _renameClasses(rightClasses, "public.kern2.")
     # populate the final group names
-    kerning = _replaceRenamedPairMembers(kerning, leftClassRename, rightClassRename)
+    kerning = _replaceRenamedPairMembers(
+        kerning, leftClassRename, rightClassRename
+    )
     leftGroups = _setGroupNames(leftClasses, leftClassRename)
     rightGroups = _setGroupNames(rightClasses, rightClassRename)
     # combine the side groups
@@ -405,6 +627,7 @@ def _extractOpenTypeKerningFromGPOS(source):
     # done.
     return kerning, groups
 
+
 def _makeScriptOrder(gpos):
     """
     Run therough GPOS and make an alphabetically
@@ -419,6 +642,7 @@ def _makeScriptOrder(gpos):
         scripts.insert(0, "DFLT")
     return sorted(scripts)
 
+
 def _gatherDataFromLookups(gpos, scriptOrder):
     """
     Gather kerning and classes from the applicable lookups
@@ -450,6 +674,7 @@ def _gatherDataFromLookups(gpos, scriptOrder):
             rightClassDictionaries.append(rightClasses)
     return kerningDictionaries, leftClassDictionaries, rightClassDictionaries
 
+
 def _gatherLookupIndexes(gpos):
     """
     Gather a mapping of script to lookup indexes
@@ -461,7 +686,11 @@ def _gatherLookupIndexes(gpos):
         }
     """
     # gather the indexes of the kern features
-    kernFeatureIndexes = [index for index, featureRecord in enumerate(gpos.FeatureList.FeatureRecord) if featureRecord.FeatureTag == "kern"]
+    kernFeatureIndexes = [
+        index
+        for index, featureRecord in enumerate(gpos.FeatureList.FeatureRecord)
+        if featureRecord.FeatureTag == "kern"
+    ]
     # find scripts and languages that have kern features
     scriptKernFeatureIndexes = {}
     for scriptRecord in gpos.ScriptList.ScriptRecord:
@@ -500,6 +729,7 @@ def _gatherLookupIndexes(gpos):
     # done
     return scriptLookupIndexes
 
+
 def _gatherKerningForLookup(gpos, lookupIndex):
     """
     Gather the kerning and class data for a particular lookup.
@@ -531,7 +761,9 @@ def _gatherKerningForLookup(gpos, lookupIndex):
                 kerning = _handleLookupType2Format1(subtable)
                 allKerning.update(kerning)
             elif (lookupType, format) == (2, 2):
-                kerning, leftClasses, rightClasses = _handleLookupType2Format2(subtable, lookupIndex, subtableIndex)
+                kerning, leftClasses, rightClasses = _handleLookupType2Format2(
+                    subtable, lookupIndex, subtableIndex
+                )
                 allKerning.update(kerning)
                 allLeftClasses.update(leftClasses)
                 allRightClasses.update(rightClasses)
@@ -543,13 +775,16 @@ def _gatherKerningForLookup(gpos, lookupIndex):
                 kerning = _handleLookupType2Format1(extSubtable)
                 allKerning.update(kerning)
             elif (lookupType, format) == (2, 2):
-                kerning, leftClasses, rightClasses = _handleLookupType2Format2(extSubtable, lookupIndex, subtableIndex)
+                kerning, leftClasses, rightClasses = _handleLookupType2Format2(
+                    extSubtable, lookupIndex, subtableIndex
+                )
                 allKerning.update(kerning)
                 allLeftClasses.update(leftClasses)
                 allRightClasses.update(rightClasses)
     # done
     return allKerning, allLeftClasses, allRightClasses
 
+
 def _handleLookupType2Format1(subtable):
     """
     Extract a kerning dictionary from a Lookup Type 2 Format 1.
@@ -571,17 +806,29 @@ def _handleLookupType2Format1(subtable):
                 kerning[leftGlyphName, rightGlyphName] = value
     return kerning
 
+
 def _handleLookupType2Format2(subtable, lookupIndex, subtableIndex):
     """
     Extract kerning, left class and right class dictionaries from a Lookup Type 2 Format 2.
     """
     # extract the classes
-    leftClasses = _extractFeatureClasses(lookupIndex=lookupIndex, subtableIndex=subtableIndex, classDefs=subtable.ClassDef1.classDefs, coverage=subtable.Coverage.glyphs)
-    rightClasses = _extractFeatureClasses(lookupIndex=lookupIndex, subtableIndex=subtableIndex, classDefs=subtable.ClassDef2.classDefs)
+    leftClasses = _extractFeatureClasses(
+        lookupIndex=lookupIndex,
+        subtableIndex=subtableIndex,
+        classDefs=subtable.ClassDef1.classDefs,
+        coverage=subtable.Coverage.glyphs,
+    )
+    rightClasses = _extractFeatureClasses(
+        lookupIndex=lookupIndex,
+        subtableIndex=subtableIndex,
+        classDefs=subtable.ClassDef2.classDefs,
+    )
     # extract the pairs
     kerning = {}
     for class1RecordIndex, class1Record in enumerate(subtable.Class1Record):
-        for class2RecordIndex, class2Record in enumerate(class1Record.Class2Record):
+        for class2RecordIndex, class2Record in enumerate(
+            class1Record.Class2Record
+        ):
             leftClass = (lookupIndex, subtableIndex, class1RecordIndex)
             rightClass = (lookupIndex, subtableIndex, class2RecordIndex)
             valueFormat1 = subtable.ValueFormat1
@@ -594,6 +841,7 @@ def _handleLookupType2Format2(subtable, lookupIndex, subtableIndex):
                 kerning[leftClass, rightClass] = value
     return kerning, leftClasses, rightClasses
 
+
 def _mergeKerningDictionaries(kerningDictionaries):
     """
     Merge all of the kerning dictionaries found into
@@ -609,6 +857,7 @@ def _mergeKerningDictionaries(kerningDictionaries):
     # done.
     return kerning
 
+
 def _findSingleMemberGroups(classDictionaries):
     """
     Find all classes that have only one member.
@@ -622,6 +871,7 @@ def _findSingleMemberGroups(classDictionaries):
                     del classDictionary[name]
     return toRemove
 
+
 def _removeSingleMemberGroupReferences(kerning, leftGroups, rightGroups):
     """
     Translate group names into glyph names in pairs
@@ -634,6 +884,7 @@ def _removeSingleMemberGroupReferences(kerning, leftGroups, rightGroups):
         new[left, right] = value
     return new
 
+
 def _mergeClasses(classDictionaries):
     """
     Look for classes that have the exact same list
@@ -669,6 +920,7 @@ def _mergeClasses(classDictionaries):
         classes[name] = members
     return classes, rename
 
+
 def _setGroupNames(classes, classRename):
     """
     Set the final names into the groups.
@@ -684,6 +936,7 @@ def _setGroupNames(classes, classRename):
         groups[groupName] = glyphList
     return groups
 
+
 def _validateClasses(classes):
     """
     Check to make sure that a glyph is not part of more than
@@ -697,7 +950,10 @@ def _validateClasses(classes):
             glyphToClass[glyphName].add(className)
     for glyphName, groupList in glyphToClass.items():
         if len(groupList) > 1:
-            raise ExtractorError("Kerning classes are in an conflicting state.")
+            raise ExtractorError(
+                "Kerning classes are in an conflicting state."
+            )
+
 
 def _replaceRenamedPairMembers(kerning, leftRename, rightRename):
     """
@@ -710,6 +966,7 @@ def _replaceRenamedPairMembers(kerning, leftRename, rightRename):
         renamedKerning[left, right] = value
     return renamedKerning
 
+
 def _renameClasses(classes, prefix):
     """
     Replace class IDs with nice strings.
@@ -717,7 +974,12 @@ def _renameClasses(classes, prefix):
     renameMap = {}
     for classID, glyphList in classes.items():
         if len(glyphList) == 0:
-            groupName = "%s_empty_lu.%d_st.%d_cl.%d" % (prefix, classID[0], classID[1], classID[2])
+            groupName = "%s_empty_lu.%d_st.%d_cl.%d" % (
+                prefix,
+                classID[0],
+                classID[1],
+                classID[2],
+            )
         elif len(glyphList) == 1:
             groupName = list(glyphList)[0]
         else:
@@ -726,7 +988,10 @@ def _renameClasses(classes, prefix):
         renameMap[classID] = groupName
     return renameMap
 
-def _extractFeatureClasses(lookupIndex, subtableIndex, classDefs, coverage=None):
+
+def _extractFeatureClasses(
+    lookupIndex, subtableIndex, classDefs, coverage=None
+):
     """
     Extract classes for a specific lookup in a specific subtable.
     This is relatively straightforward, except for class 0 interpretation.
@@ -761,12 +1026,15 @@ def _extractFeatureClasses(lookupIndex, subtableIndex, classDefs, coverage=None)
         classes[lookupIndex, subtableIndex, classIndex] = frozenset(glyphList)
     return classes
 
+
 def _extractOpenTypeKerningFromKern(source):
     kern = source["kern"]
     kerning = {}
     for subtable in kern.kernTables:
         if subtable.version != 0:
-            raise ExtractorError("Unknown kern table formst: %d" % subtable.version)
+            raise ExtractorError(
+                "Unknown kern table formst: %d" % subtable.version
+            )
         # XXX the spec defines coverage values for
         # kerning direction (horizontal or vertical)
         # minimum (some sort of kerning restriction)
